@@ -1,6 +1,6 @@
 ################################################################################
 # "Working with dynamic models for agriculture"
-# R script for pratical work
+# R script for practical work
 # Daniel Wallach (INRA), David Makowski (INRA), James W. Jones (U.of Florida),
 # Francois Brun (ACTA)
 # version : 2010-08-09
@@ -10,20 +10,36 @@
 #' @description \strong{Model description.}
 #' This model is a dynamic model of crop growth for Maize cultivated in potential conditions.
 #' The crop growth is represented by three state variables, leaf area per unit ground area (leaf area index, LAI), total biomass (B) and cumulative thermal time since plant emergence (TT). It is based on key concepts included in most crop models, at least for the "potential production" part. In fact, this model does not take into account any effects of soil water, nutrients, pests, or diseases,... 
-#' The tree state variables are dynamic variables depending on days after emergence: TT(day), B(day), and LAI(day). The model has a time step dt of one day.\cr
+#' @details The tree state variables are dynamic variables depending on days after emergence: TT(day), B(day), and LAI(day). The model has a time step dt of one day.\cr
 #' The model is defined by a few equations, with a total of seven parameters for the described process.
-#' \cr (1) \eqn{TT(day  +  1)  =  TT(day )  +  dTT(day)}
-#' @param Tbase,RUE,K,alpha,LAImax,TTM,TTL : 7 parameters
+#' \cr (1) \eqn{TT(day+1) = TT(day)+dTT(day)}{TT(day+1) = TT(day)+dTT(day)}
+#' \cr (2) \eqn{B(day+1) = B(day)+dB(day)}{B(day+1) = B(day)+dB(day)}
+#' \cr (3) \eqn{LAI(day+1) = LAI(day)+dLAI(day)}{LAI(day+1) = LAI(day)+dLAI(day)}
+#' \cr (4) \eqn{dTT(day) = \max(\frac{TMIN(day)+TMAX(day)}{2}-Tbase;0)}{dTT(day) = max((TMIN(day)+TMAX(day))/2-Tbase ; 0)}
+#' \cr (5) \eqn{dB(day) = RUE*(1-e^{-K*LAI(day)*I(day)}),\ if\ TT(day)\le TTM}{dB(day) = RUE*(1-e^{-K*LAI(day)*I(day)}), if TT(day)<= TTM} 
+#' \cr \eqn{dB(day) = 0,\ if\  TT(day)>TTM}{dB(day) = 0, if TT(day)>TTM}
+#' \cr (6) \eqn{dLAI(day) = alpha*dTT(day)*LAI(day)*\max(LAImax-LAI(day);0),\ if \ TT(day)\le TTL }{alpha*dTT(day)*LAI(day)*max(LAImax-LAI(day);0), if TT(day)<= TTL }
+#' \cr \eqn{dLAI(day) = 0,\ if\  TT(day)>TTL }{dLAI(day) = 0 if TT(day)>TTL}
+#' @param Tbase :	parameter the baseline temperature for growth (degreeCelsius)
+#' @param TTM	:	parameter temperature sum for crop maturity (degreeC.day)
+#' @param TTL	:	parameter temperature sum at the end of leaf area increase (degreeC.day)
+#' @param K : parameter extinction coefficient (relation between leaf area index and intercepted radiation) (-)
+#' @param RUE : parameter radiation use efficiency (?)
+#' @param alpha : parameter the relative rate of leaf area index increase for small values of leaf area index (?)
+#' @param LAImax : parameter maximum leaf area index (-)
 #' @param weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
 #' @return data.frame with daily TT, LAI,B
 #' @export
+#' @examples 
+#' weather = maize.weather(working.year=2010, working.site=30,weather_all=weather_EuropeEU)
+#' maize.model(Tbase=7, RUE=1.85, K=0.7, alpha=0.00243, LAImax=7, TTM=1200, TTL=700, weather, sdate=100, ldate=250)
 maize.model<-function(Tbase,RUE,K,alpha,LAImax,TTM,TTL,weather,sdate,ldate)
     {
     # Initialize variables
     # 3 states variables, as 3 vectors initialized to NA
-    # TT : temperature sum (°C.d)
+    # TT : temperature sum (degreeC.day)
     TT<-rep(NA,ldate)
     # B : Biomass (g/m2)
     B<-rep(NA,ldate)
@@ -45,7 +61,7 @@ maize.model<-function(Tbase,RUE,K,alpha,LAImax,TTM,TTL,weather,sdate,ldate)
         if (TT[day]<=TTL) {dLAI <- alpha*dTT*LAI[day]*max(LAImax-LAI[day],0)}
         else {dLAI <-0 }
         
-        # Uptade state variables 
+        # Update state variables 
         TT[day+1]<- TT[day] + dTT 
         B[day+1]<- B[day] + dB
         LAI[day+1]<- LAI[day] + dLAI   
@@ -54,54 +70,28 @@ maize.model<-function(Tbase,RUE,K,alpha,LAImax,TTM,TTL,weather,sdate,ldate)
     return(data.frame(day=sdate:ldate,TT=TT[sdate:ldate],LAI=LAI[sdate:ldate],B=B[sdate:ldate]))    
     }
 ################################################################################
-#' @title The basic Maize model function with a litle change to use with maize.simul
+#' @title The basic Maize model function with a little change to use with maize.simul
 #' @param param : a vector of parameters
 #' @param weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
 #' @return data.frame with daily TT, LAI,B
 #' @export
+#' @examples 
+#' weather = maize.weather(working.year=2010, working.site=30,weather_all=weather_EuropeEU)
+#' maize.model2(maize.define.param()["nominal",], weather, sdate=100, ldate=250)
 maize.model2<-function(param, weather,sdate,ldate)
 {
-# 7 Parameter values of the model, read from the param vector
-Tbase <- param["Tbase"]
-RUE <- param["RUE"]
-K <- param["K"]
-alpha <- param["alpha"]
-LAImax <- param["LAImax"]
-TTM <- param["TTM"]
-TTL <- param["TTL"]
-    # Initialize variables
-    # 3 states variables, as 3 vectors initialized to NA
-    # TT : temperature sum (°C.d)
-    TT<-rep(NA,ldate)
-    # B : Biomass (g/m2)
-    B<-rep(NA,ldate)
-    # LAI : Leaf Area Index (m2 leaf/m2 soil)
-    LAI<-rep(NA,ldate)
-
-    # Initialize state variables when sowing on day "sdate"
-    TT[sdate]<- 0
-    B[sdate]<- 1
-    LAI[sdate]<- 0.01
-
-    # Simulation loop
-    for (day in sdate:(ldate-1))
-        {
-        # Calculate rates of change of state variables (dTT, dB, dLAI)
-        dTT <- max((weather$Tmin[day]+weather$Tmax[day])/2-Tbase, 0)
-        if (TT[day]<=TTM) {dB <- RUE*(1-exp(-K*LAI[day]))*weather$I[day]}
-        else {dB <- 0}
-        if (TT[day]<=TTL) {dLAI <- alpha*dTT*LAI[day]*max(LAImax-LAI[day],0)}
-        else {dLAI <-0 }
-
-        # Uptade state variables
-        TT[day+1]<- TT[day] + dTT
-        B[day+1]<- B[day] + dB
-        LAI[day+1]<- LAI[day] + dLAI
-        }
-        # End simulation loop
-    return(data.frame(day=sdate:ldate,TT=TT[sdate:ldate],LAI=LAI[sdate:ldate],B=B[sdate:ldate]))
+  # 7 Parameter values of the model, read from the param vector
+  Tbase <- param["Tbase"]
+  RUE <- param["RUE"]
+  K <- param["K"]
+  alpha <- param["alpha"]
+  LAImax <- param["LAImax"]
+  TTM <- param["TTM"]
+  TTL <- param["TTL"]
+  # use maize.model function to run the model
+  return(maize.model(Tbase, RUE, K, alpha, LAImax, TTM, TTL, weather, sdate, ldate))
 }
 ################################################################################
 #' @title Define parameter values of the maize model
@@ -110,31 +100,31 @@ TTL <- param["TTL"]
 maize.define.param <- function()
 {
 # nominal, binf, bsup
-# Tbase  : the baseline temperature for growth (°C)
+# Tbase  : the baseline temperature for growth (degreeC)
 Tbase <- c(7, 6, 8)
 # RUE : radiation use efficiency (g.MJ-1)
 RUE <- c(1.85,1.5,2.5)
 # K : extinction coefficient (-)
 K <- c(0.7,0.6,0.8)
-#alpha : the relative rate of leaf area index increase for small values of leaf area index ((°C.day)-1)
+#alpha : the relative rate of leaf area index increase for small values of leaf area index ((degreeC.day)-1)
 alpha <- c(0.00243,0.002,0.003)
 #LAImax : maximum leaf area index (m2 leaf/m2 soil)
 LAImax <- c(7.0,6.0,8.0)
-#TTM :  temperature sum for crop maturity (°C.day)
+#TTM :  temperature sum for crop maturity (degreeC.day)
 TTM <- c(1200,1100,1400)
-#TTL : temperature sum at the end of leaf area increase (°C.day)
+#TTL : temperature sum at the end of leaf area increase (degreeC.day)
 TTL <- c(700,600,850)
 param<-data.frame(Tbase,RUE,K, alpha, LAImax, TTM, TTL)
 row.names(param)<-c("nominal","binf","bsup")
 return(as.matrix(param))
 }
 ################################################################################
-#' @title Wrapping fonction to run an virtual experimental design for maize model
+#' @title Wrapping function to run an virtual experimental design for maize model
 #' @param X : matrix of n row vectors of 7 parameters
 #' @param weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
-#' @param all : if you want a matrix combining X and output (delfaut = FALSE)
+#' @param all : if you want a matrix combining X and output (default = FALSE)
 #' @return matrix with maximum biomass for each parameter vector
 #' @export
 maize.simule <- function(X, weather, sdate, ldate, all=FALSE){
@@ -144,8 +134,76 @@ if(all) Y = cbind(X,B = Y)
 return(as.matrix(Y))
 }
 ################################################################################
-#' @title the Maize model function with additionnal state varaiable CumInt
-#' @param Tbase,RUE,K,alpha,LAImax,TTM,TTL : 7 parameters
+#' @title Wrapping function to run maize model on several site-years
+#' @param param : a vector of parameters
+#' @param list_site_year : vector of site-year
+#' @param sdate : sowing date
+#' @param ldate : last date
+#' @param weather_all : weather data.frame for corresponding site-years
+#' @return a data.frame with simulation for all site-years, with the first column sy indicating the site-years
+#' @export
+maize.multisy<-function(param, list_site_year, sdate,ldate, weather_all=weather_EuropeEU){
+  sim <- data.frame()
+  for(sy in list_site_year){
+    weather = maize.weather(working.year=strsplit(sy,"-")[[1]][2], working.site=strsplit(sy,"-")[[1]][1],weather_all=weather_all)
+    result <- maize.model2(param, weather, sdate, ldate)
+    result <- cbind(sy,result)
+    sim <- rbind(sim, result)
+    }
+  return(sim)
+}
+################################################################################
+#' @title Wrapping function to run maize model on a matrix of parameter (virtual design) and give Biomass at day240
+#' @param X : matrix of n row vectors of 7 parameters
+#' @param weather : weather data.frame for one single year
+#' @param sdate : sowing date
+#' @param ldate : last date
+#' @param all : if you want a matrix combining X and output (default = FALSE)
+#' @return a matrix of biomass at day=240 for all combinations of parameters of X
+#' @export
+#' @examples sy="18-2006"
+#' weather = maize.weather(working.year=strsplit(sy,"-")[[1]][2], working.site=strsplit(sy,"-")[[1]][1],weather_all=weather_EuropeEU)
+#' maize.simule240(maize.define.param(),weather, sdate=100, ldate=250, all=FALSE)
+maize.simule240<-function(X, weather, sdate, ldate, all=FALSE){
+Y <- apply(X,1,function(v) maize.model2(v[1:7],weather, sdate, ldate)[240-sdate+1,"B"])
+if(all) Y = cbind(X,B = Y)
+return(as.matrix(Y))}
+################################################################################
+#' @title Wrapping function to run maize model on a list of site-year and compute mean Biomass at day240.
+#' @param param : a vector of parameters
+#' @param liste_sy : vector of site-year
+#' @param sdate : sowing date
+#' @param ldate : last date
+#' @return mean biomass at day=240
+#' @export
+#' @examples maize.multisy240(maize.define.param()["nominal",],c("18-2006","64-2004") , sdate=100, ldate=250)
+maize.multisy240<-function(param,liste_sy, sdate, ldate){
+Y <- sapply(liste_sy,function(sy) maize.model2(param,maize.weather(working.year=strsplit(sy,"-")[[1]][2], working.site=strsplit(sy,"-")[[1]][1],weather_all=weather_EuropeEU),sdate,ldate)[240-sdate+1,"B"])
+return(mean(as.matrix(Y)))}
+################################################################################
+#' @title Wrapping function to run maize model and compute mean Biomass at day240 for a list of site-year, for a matrix of parameter (design).
+#' @param X : matrix of n row vectors of 7 parameters
+#' @param liste_sy : vector of site-year
+#' @param sdate : sowing date
+#' @param ldate : last date
+#' @param all : if you want a matrix combining X and output (default = FALSE)
+#' @return a matrix of mean biomass at day=240 for all combinations of parameters of X
+#' @export
+#' @examples maize.simule_multisy240(maize.define.param(),c("18-2006","64-2004") , sdate=100, ldate=250, all=FALSE)
+maize.simule_multisy240<-function(X,liste_sy, sdate, ldate, all=FALSE){
+Y <- apply(X,1,function(v) maize.multisy240(v[1:7],liste_sy, sdate, ldate))
+if(all) Y = cbind(X,B = Y)
+return(as.matrix(Y))
+}
+################################################################################
+#' @title the Maize model function with additional state variable CumInt
+#' @param Tbase :	parameter the baseline temperature for growth (degreeCelsius)
+#' @param TTM	:	parameter temperature sum for crop maturity (degreeC.day)
+#' @param TTL	:	parameter temperature sum at the end of leaf area increase (degreeC.day)
+#' @param K : parameter extinction coefficient (relation between leaf area index and intercepted radiation) (-)
+#' @param RUE : parameter radiation use efficiency (?)
+#' @param alpha : parameter the relative rate of leaf area index increase for small values of leaf area index (?)
+#' @param LAImax : parameter maximum leaf area index (-)
 #' @param  weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
@@ -155,7 +213,7 @@ maize_cir.model<-function(Tbase,RUE,K,alpha,LAImax,TTM,TTL,weather,sdate,ldate)
     {
     # Initialize variables
     # 3 states variables, as 3 vectors initialized to NA
-    # TT : temperature sum (°C.d)
+    # TT : temperature sum (degreeC.d)
     TT<-rep(NA,ldate)
     # B : Biomass (g/m2)
     B<-rep(NA,ldate)
@@ -182,7 +240,7 @@ maize_cir.model<-function(Tbase,RUE,K,alpha,LAImax,TTM,TTL,weather,sdate,ldate)
         else {dLAI <-0 }
 		
         
-        # Uptade state variables 
+        # Update state variables 
         TT[day+1]<- TT[day] + dTT 
         B[day+1]<- B[day] + dB
         LAI[day+1]<- LAI[day] + dLAI   
@@ -208,8 +266,14 @@ maize.RUEtemp <- function(T, RUE_max,T0,T1,T2,T3)
 	}
 ###############################################################################
 #' @title The Maize model function with rue (and CumInt)
-#' @param Tbase,RUE_max,K,alpha,LAImax,TTM,TTL : 7 parameters, weather table, sowing date and last date
-#' @param  weather : weather data.frame for one single year
+#' @param Tbase :	parameter the baseline temperature for growth (degreeCelsius)
+#' @param TTM	:	parameter temperature sum for crop maturity (degreeC.day)
+#' @param TTL	:	parameter temperature sum at the end of leaf area increase (degreeC.day)
+#' @param K : parameter extinction coefficient (relation between leaf area index and intercepted radiation) (-)
+#' @param RUE_max : parameter maximum radiation use efficiency (?)
+#' @param alpha : parameter the relative rate of leaf area index increase for small values of leaf area index (?)
+#' @param LAImax : parameter maximum leaf area index (-)
+#' @param weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
 #' @return data.frame with daily TT, LAI,B
@@ -218,7 +282,7 @@ maize_cir_rue.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,sdate
     {
     # Initialize variables
     # 3 states variables, as 3 vectors initialized to NA
-    # TT : temperature sum (°C.d)
+    # TT : temperature sum (degreeC.d)
     TT<-rep(NA,ldate)
     # B : Biomass (g/m2)
     B<-rep(NA,ldate)
@@ -226,14 +290,11 @@ maize_cir_rue.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,sdate
     LAI<-rep(NA,ldate)
     # CumInt LAI : Cumulative intercepted radiation
 	CumInt<-rep(NA,ldate) 
-
-
     # Initialize state variables when sowing on day "sdate"   
     TT[sdate]<- 0
     B[sdate]<- 1
     LAI[sdate]<- 0.01
     CumInt[sdate] = 0.0
-
     # Simulation loop
     for (day in sdate:(ldate-1))
         {
@@ -244,9 +305,8 @@ maize_cir_rue.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,sdate
         else {dB <- 0}
         if (TT[day]<=TTL) {dLAI <- alpha*dTT*LAI[day]*max(LAImax-LAI[day],0)}
         else {dLAI <-0 }
-		
         
-        # Uptade state variables 
+        # Update state variables 
         TT[day+1]<- TT[day] + dTT 
         B[day+1]<- B[day] + dB
         LAI[day+1]<- LAI[day] + dLAI   
@@ -258,7 +318,13 @@ maize_cir_rue.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,sdate
 
 ###############################################################################
 #' @title The Maize model function with CumInt, RUE and ear growth
-#' @param Tbase,RUE_max,K,alpha,LAImax,TTM,TTL : 7 parameters, weather table, sowing date and last date
+#' @param Tbase :	parameter the baseline temperature for growth (degreeCelsius)
+#' @param TTM	:	parameter temperature sum for crop maturity (degreeC.day)
+#' @param TTL	:	parameter temperature sum at the end of leaf area increase (degreeC.day)
+#' @param K : parameter extinction coefficient (relation between leaf area index and intercepted radiation) (-)
+#' @param RUE_max : parameter maximum radiation use efficiency (?)
+#' @param alpha : parameter the relative rate of leaf area index increase for small values of leaf area index (?)
+#' @param LAImax : parameter maximum leaf area index (-)
 #' @param  weather : weather data.frame for one single year
 #' @param sdate : sowing date
 #' @param ldate : last date
@@ -268,7 +334,7 @@ maize_cir_rue_ear.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,s
     {
     # Initialize variables
     # 3 states variables, as 3 vectors initialized to NA
-    # TT : temperature sum (°C.d)
+    # TT : temperature sum (degreeC.d)
     TT<-rep(NA,ldate)
     # B : Biomass (g/m2)
     B<-rep(NA,ldate)
@@ -302,7 +368,7 @@ maize_cir_rue_ear.model<-function(Tbase,RUE_max,K,alpha,LAImax,TTM,TTL,weather,s
 		else dBE <-0			
 		
         
-        # Uptade state variables 
+        # Update state variables 
         TT[day+1]<- TT[day] + dTT 
         B[day+1]<- B[day] + dB
         LAI[day+1]<- LAI[day] + dLAI   
@@ -325,9 +391,9 @@ maize.weather <- function(working.year=NA, working.site=NA,weather_all=weather_F
     {
     # WEYR => year
     # WEDAY => day
-    # SRAD => I: solar radiation (MJ)
-    # TMAX => Tmax : maximum temperature (°C)
-    # TMIN => Tmin : minimum temperature (°C)
+    # SRAD => I, solar radiation (MJ)
+    # TMAX => Tmax, maximum temperature (degreeC)
+    # TMIN => Tmin, minimum temperature (degreeC)
     # select only useful header
     weather=weather_all[,c("idsite","GPSlatitude","GPSlongitude","WEYR","WEDAY","SRAD","TMAX","TMIN")]
     names(weather)[names(weather)=="WEDAY"]= "day"
